@@ -22,6 +22,8 @@
 #include <cmath>
 #include <vector>
 #include <iostream>
+#include <sstream>
+#include <unordered_map>
 
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/Coordinate.h>
@@ -42,7 +44,7 @@ using namespace geos::geom;
 
 
 VoronoiDiagramBuilder::VoronoiDiagramBuilder() :
-    tolerance(0.0), clipEnv(nullptr)
+    tolerance(0.0), clipEnv(nullptr), inputGeom(nullptr), inputSeq(nullptr), isOrdered(false)
 {
 }
 
@@ -50,12 +52,14 @@ void
 VoronoiDiagramBuilder::setSites(const geom::Geometry& geom)
 {
     siteCoords = DelaunayTriangulationBuilder::extractUniqueCoordinates(geom);
+    inputGeom = &geom;
 }
 
 void
 VoronoiDiagramBuilder::setSites(const geom::CoordinateSequence& coords)
 {
     siteCoords = DelaunayTriangulationBuilder::unique(&coords);
+    inputSeq = &coords;
 }
 
 void
@@ -64,6 +68,12 @@ VoronoiDiagramBuilder::setClipEnvelope(const geom::Envelope* nClipEnv)
     clipEnv = nClipEnv;
 }
 
+void
+VoronoiDiagramBuilder::setOrdered(bool p_isOrdered)
+{
+    isOrdered = p_isOrdered;
+}
+
 void
 VoronoiDiagramBuilder::setTolerance(double nTolerance)
 {
@@ -105,6 +115,15 @@ VoronoiDiagramBuilder::getSubdivision()
     return std::move(subdiv);
 }
 
+std::size_t
+VoronoiDiagramBuilder::getNumInputPoints() const {
+    if (inputGeom) {
+        return inputGeom->getNumPoints();
+    } else {
+        return inputSeq->getSize();
+    }
+}
+
 std::unique_ptr<geom::GeometryCollection>
 VoronoiDiagramBuilder::getDiagram(const geom::GeometryFactory& geomFact)
 {
@@ -113,6 +132,17 @@ VoronoiDiagramBuilder::getDiagram(const geom::GeometryFactory& geomFact)
     std::unique_ptr<GeometryCollection> ret;
     if (subdiv) {
         auto polys = subdiv->getVoronoiCellPolygons(geomFact);
+
+        if (isOrdered) {
+            reorderCellsToInput(polys);
+        }
+
+        for (auto& p : polys) {
+            // Don't let references to Vertex objects
+            // owned by the QuadEdgeSubdivision escape
+            p->setUserData(nullptr);
+        }
+
         ret = clipGeometryCollection(polys, diagramEnv);
     }
 
@@ -123,7 +153,7 @@ VoronoiDiagramBuilder::getDiagram(const geom::GeometryFactory& geomFact)
     return ret;
 }
 
-std::unique_ptr<geom::Geometry>
+std::unique_ptr<MultiLineString>
 VoronoiDiagramBuilder::getDiagramEdges(const geom::GeometryFactory& geomFact)
 {
     create();
@@ -132,13 +162,29 @@ VoronoiDiagramBuilder::getDiagramEdges(const geom::GeometryFactory& geomFact)
         return geomFact.createMultiLineString();
     }
 
-    std::unique_ptr<geom::MultiLineString> edges = subdiv->getVoronoiDiagramEdges(geomFact);
+    auto edges = subdiv->getVoronoiDiagramEdges(geomFact);
+
     if(edges->isEmpty()) {
-        return std::unique_ptr<Geometry>(edges.release());
+        return edges;
     }
+
     std::unique_ptr<geom::Geometry> clipPoly(geomFact.toGeometry(&diagramEnv));
     std::unique_ptr<Geometry> clipped(clipPoly->intersection(edges.get()));
-    return clipped;
+
+    switch (clipped->getGeometryTypeId()) {
+        case GEOS_LINESTRING: {
+            std::vector<std::unique_ptr<LineString>> lines;
+            lines.emplace_back(static_cast<LineString*>(clipped.release()));
+            return geomFact.createMultiLineString(std::move(lines));
+        }
+        case GEOS_MULTILINESTRING: {
+            std::unique_ptr<MultiLineString> mls(static_cast<MultiLineString*>(clipped.release()));
+            return mls;
+        }
+        default: {
+            throw util::GEOSException("Unknown state");
+        }
+    }
 }
 
 std::unique_ptr<geom::GeometryCollection>
@@ -157,10 +203,8 @@ VoronoiDiagramBuilder::clipGeometryCollection(std::vector<std::unique_ptr<Geomet
         // don't clip unless necessary
         if(clipEnv.contains(g->getEnvelopeInternal())) {
             clipped.push_back(std::move(g));
-            // TODO: check if userData is correctly cloned here?
         } else if(clipEnv.intersects(g->getEnvelopeInternal())) {
             auto result = clipPoly->intersection(g.get());
-            result->setUserData(g->getUserData()); // TODO: needed ?
             if (!result->isEmpty()) {
                 clipped.push_back(std::move(result));
             }
@@ -170,5 +214,69 @@ VoronoiDiagramBuilder::clipGeometryCollection(std::vector<std::unique_ptr<Geomet
     return gfact->createGeometryCollection(std::move(clipped));
 }
 
+void
+VoronoiDiagramBuilder::addCellsForCoordinates(CoordinateCellMap& cellMap,
+                                              const CoordinateSequence& seq,
+                                              std::vector<std::unique_ptr<Geometry>> & polys) {
+    for (const CoordinateXY& c : seq.items<CoordinateXY>()) {
+        auto cell = cellMap.find(c);
+
+        if (cell == cellMap.end()) {
+            std::stringstream ss;
+            ss << "No cell found for input coordinate " << c;
+            throw util::GEOSException(ss.str());
+        }
+
+        if (cell->second == nullptr) {
+            std::stringstream ss;
+            ss << "Multiple input coordinates in cell at " << c;
+            throw util::GEOSException(ss.str());
+        }
+
+        polys.push_back(std::move(cell->second));
+    }
+}
+
+void
+VoronoiDiagramBuilder::addCellsForCoordinates(CoordinateCellMap& cellMap,
+                                              const Geometry& g,
+                                              std::vector<std::unique_ptr<Geometry>> & polys) {
+    auto typ = g.getGeometryTypeId();
+
+    if (typ == GEOS_LINESTRING) {
+        const auto& seq = *static_cast<const LineString&>(g).getCoordinatesRO();
+        addCellsForCoordinates(cellMap, seq, polys);
+    } else if (typ == GEOS_POINT) {
+        const auto& seq = *static_cast<const Point&>(g).getCoordinatesRO();
+        addCellsForCoordinates(cellMap, seq, polys);
+    } else {
+        for (std::size_t i = 0; i < g.getNumGeometries(); i++) {
+            addCellsForCoordinates(cellMap, *g.getGeometryN(i), polys);
+        }
+    }
+}
+
+void
+VoronoiDiagramBuilder::reorderCellsToInput(std::vector<std::unique_ptr<Geometry>> & polys) const
+{
+    CoordinateCellMap cellMap;
+    for (auto& p : polys) {
+        const CoordinateXY* c = reinterpret_cast<const Coordinate*>(p->getUserData());
+        cellMap.emplace(*c, std::move(p));
+    }
+
+    auto npts = getNumInputPoints();
+    std::vector<std::unique_ptr<Geometry>> reorderedPolys;
+    reorderedPolys.reserve(npts);
+
+    if (inputSeq) {
+        addCellsForCoordinates(cellMap, *inputSeq, reorderedPolys);
+    } else {
+        addCellsForCoordinates(cellMap, *inputGeom, reorderedPolys);
+    }
+
+    polys = std::move(reorderedPolys);
+}
+
 } //namespace geos.triangulate
 } //namespace geos
