@@ -16,7 +16,6 @@
 #include <geos/profiler.h>
 #include <geos/constants.h>
 #include <geos/geom/Coordinate.h>
-#include <geos/geom/CoordinateArraySequence.h>
 #include <geos/geom/CoordinateSequence.h>
 #include <geos/geom/Geometry.h>
 #include <geos/geom/GeometryFactory.h>
@@ -66,13 +65,12 @@ struct test_facetdistanceop_data {
         using geos::operation::distance::IndexedFacetDistance;
         GeomPtr g1(_wktreader.read(wkt1));
         GeomPtr g2(_wktreader.read(wkt2));
-        std::vector<geos::geom::Coordinate> pts;
-        pts = IndexedFacetDistance::nearestPoints(g1.get(), g2.get());
-        ensure(fabs(pts[0].distance(pts[1])-distance) < 1e08);
-        ensure(fabs(pts[0].x - p1.x) < 1e-08);
-        ensure(fabs(pts[0].y - p1.y) < 1e-08);
-        ensure(fabs(pts[1].x - p2.x) < 1e-08);
-        ensure(fabs(pts[1].y - p2.y) < 1e-08);
+        auto pts = IndexedFacetDistance::nearestPoints(g1.get(), g2.get());
+        ensure(fabs((*pts)[0].distance((*pts)[1])-distance) < 1e08);
+        ensure(fabs((*pts)[0].x - p1.x) < 1e-08);
+        ensure(fabs((*pts)[0].y - p1.y) < 1e-08);
+        ensure(fabs((*pts)[1].x - p2.x) < 1e-08);
+        ensure(fabs((*pts)[1].y - p2.y) < 1e-08);
         return;
     }
 
@@ -93,16 +91,16 @@ struct test_facetdistanceop_data {
     std::unique_ptr<geos::geom::LineString>
     makeSinCircle(std::size_t nvertices, double radius, double amplitude)
     {
-        geos::geom::CoordinateArraySequence cs;
+        auto cs = geos::detail::make_unique<geos::geom::CoordinateSequence>();
         std::vector<geos::geom::Coordinate> coords;
         for (std::size_t i = 0; i < nvertices; i++) {
             geos::geom::Coordinate c;
             double angle = (double)i*360.0/(double)nvertices;
             angle2sincircle(angle, radius, amplitude, &c.x, &c.y);
-            cs.add(c);
+            cs->add(c);
         }
 
-        std::unique_ptr<geos::geom::LineString> ls(_factory->createLineString(cs));
+        auto ls = _factory->createLineString(std::move(cs));
         return ls;
     }
 
@@ -242,8 +240,13 @@ void object::test<8>
     GeomPtr g0(_wktreader.read(wkt0));
     GeomPtr g1(_wktreader.read(wkt1));
     IndexedFacetDistance ifd(g0.get());
-    double d = ifd.distance(g1.get());
-    ensure_equals(d, 50);
+    double expected_distance = 50;
+
+    double actual_distance = ifd.distance(g1.get());
+    ensure_equals(actual_distance, expected_distance);
+
+    ensure(ifd.isWithinDistance(g1.get(), expected_distance));
+    ensure(!ifd.isWithinDistance(g1.get(), expected_distance - 1e-6));
 }
 
 // Invalid polygon collapsed to a line
@@ -261,9 +264,9 @@ void object::test<9>
     double d = ifd.distance(g1.get());
     ensure_equals("incorrect distance", d, 0.0, 0.001);
 
-    std::vector<geos::geom::Coordinate> nearestPts = ifd.nearestPoints(g1.get());
-    ensure_equals("nearest points x", nearestPts[0].x, nearestPts[1].x, 0.00001);
-    ensure_equals("nearest points y", nearestPts[0].y, nearestPts[1].y, 0.00001);
+    auto nearestPts = ifd.nearestPoints(g1.get());
+    ensure_equals("nearest points x", (*nearestPts)[0].x, (*nearestPts)[1].x, 0.00001);
+    ensure_equals("nearest points y", (*nearestPts)[0].y, (*nearestPts)[1].y, 0.00001);
 }
 
 // Invalid polygon collapsed to a line
@@ -392,7 +395,7 @@ void object::test<12>()
     IndexedFacetDistance ifd1(g1.get());
 
     auto pts = ifd1.nearestPoints(g2.get());
-    ensure_equals(pts.size(), 2u);
+    ensure_equals(pts->size(), 2u);
 
     auto ls = _factory->createLineString(std::move(pts));
     ls->normalize();
