@@ -237,7 +237,7 @@ template<>
 void object::test<9>
 ()
 {
-    std::string wkt_in("MULTIPOINT(80 200, 240 200, 240 60, 80 60, 80 200, 140 199, 120 120)");
+    std::string wkt_in("MULTIPOINT((80 200), (240 200), (240 60), (80 60), (80 200), (140 199), (120 120))");
 
     GeomPtr g(wktreader.read(wkt_in));
 
@@ -278,7 +278,7 @@ void object::test<11>
 ()
 {
     std::string wkt_in("GEOMETRYCOLLECTION ( \
-					MULTIPOINT (80 200, 240 200, 240 60, 80 60, 80 200, 140 199, 120 120), \
+					MULTIPOINT ((80 200), (240 200), (240 60), (80 60), (80 200), (140 199), (120 120)), \
 					POLYGON ((80 200, 240 200, 240 60, 80 60, 80 200)), \
 					LINESTRING (80 200, 240 200, 240 60, 80 60, 80 200, 140 199, 120 120) )");
 
@@ -308,7 +308,7 @@ void object::test<12>
     std::string
     wkt("MULTIPOLYGON(((0.561648 1,1 1,1 0,0.468083 0,0.52758 0.00800554,0.599683 0.0280924, 0.601611 0.265374, \
                         0.622693 0.316765,0.69507 0.357497,0.695623 0.429711,0.655111 0.502298, 0.696467 0.543147,0.840712 0.593546, \
-                        0.882583 0.66546,0.852357 0.748213,0.84264 0.789567,0 .832667 0.841202,0.832667 0.841202,0.740538 0.873004, \
+                        0.882583 0.66546,0.852357 0.748213,0.84264 0.789567,0 .832667,0.832667 0.841202,0.740538 0.873004, \
                         0.617349 0.905045,0.566576 0.977697,0.561648 1)),((0 0.801979,0.0308575 0.786234,0.0705513 0.631135, \
                         0.141616 0.527248,0.233985 0.505872,0.264777 0.526263,0.336631 0.505009,0.356603 0.422321,0.355803 0.350038, \
                         0.375252 0.205364,0.415206 0.0709182,0.45479 0,0 0,0 0,0 0.801979)))");
@@ -432,5 +432,28 @@ void object::test<15>
     ensure_equals_geometry(simplified.get(), expected.get());
 }
 
+// Test that start point of a polygon can be removed
+template<>
+template<>
+void object::test<16>
+()
+{
+     auto g = wktreader.read("POLYGON ((1 0, 2 0, 2 2, 0 2, 0 0, 1 0))");
+     auto simplified = DouglasPeuckerSimplifier::simplify(g.get(), 0);
+     auto expected = wktreader.read("POLYGON (( 0 0, 2 0, 2 2, 0 2, 0 0))");
+     ensure_equals_geometry(simplified.get(), expected.get());
+}
+
+// Test that start point of a closed LineString is not changed
+template<>
+template<>
+void object::test<17>
+()
+{
+     auto g = wktreader.read("LINESTRING (1 0, 2 0, 2 2, 0 2, 0 0, 1 0)");
+     auto simplified = DouglasPeuckerSimplifier::simplify(g.get(), 0);
+     ensure_equals_geometry(simplified.get(), g.get());
+}
+
 
 } // namespace tut
